@@ -16,11 +16,11 @@ CGI::Buffer - Verify, Cache and Optimise CGI Output
 
 =head1 VERSION
 
-Version 0.78
+Version 0.79
 
 =cut
 
-our $VERSION = '0.78';
+our $VERSION = '0.79';
 
 =head1 SYNOPSIS
 
@@ -84,6 +84,7 @@ our $cache_age;
 our $cache_key;
 our $info;
 our $logger;
+our $lingua;
 our $status;
 our $script_mtime;
 our $cobject;
@@ -247,6 +248,7 @@ END {
 			}
 			my $lint = HTML::Lint->new();
 			$lint->parse($body);
+			$lint->eof();
 
 			if($lint->errors) {
 				$headers = 'Status: 500 Internal Server Error';
@@ -315,6 +317,7 @@ END {
 					$body = substr($body, 0, $2);
 				}
 				$unzipped_body = $body;
+				$status = 206;
 			}
 		}
 		_compress({ encoding => $encoding });
@@ -556,10 +559,16 @@ END {
 		# CGI::Buffer
 		unshift @o, split(/\r\n/, $headers);
 		if($body && $send_body) {
-			unless(grep(/^Content-Length: /, @o)) {
+			unless(grep(/^Content-Length: \d/, @o)) {
 				push @o, "Content-Length: $body_length";
 			}
 		}
+		unless(grep(/^Status: \d/, @o)) {
+			require HTTP::Status;
+			HTTP::Status->import();
+
+			push @o, "Status: $status " . HTTP::Status::status_message($status);
+		}
 	} else {
 		push @o, "X-CGI-Buffer-$VERSION: No headers";
 	}
@@ -685,6 +694,7 @@ sub _optimise_content {
 	$body =~ s/\<\/tr\>\s\<\/table\>/\<\/tr\>\<\/table\>/gi;
 	$body =~ s/\<br\s?\/?\>\s?\<p\>/\<p\>/gi;
 	$body =~ s/\<br\>\s/\<br\>/gi;
+	$body =~ s/\s+\<br\>/\<br\>/gi;
 	$body =~ s/\<br\s?\/\>\s/\<br \/\>/gi;
 	$body =~ s/[ \t]+/ /gs;	# Remove duplicate space, don't use \s+ it breaks JavaScript
 	$body =~ s/\s\<p\>/\<p\>/gi;
@@ -714,9 +724,10 @@ sub _generate_key {
 		$info = CGI::Info->new({ cache => $cache });
 	}
 
-	# TODO: Use CGI::Lingua so that different languages are stored
-	#	in different caches
 	my $key = $info->browser_type() . '::' . $info->domain_name() . '::' . $info->script_name() . '::' . $info->as_string();
+	if($lingua) {
+		$key .= '::' . $lingua->language();
+	}
 	if($ENV{'HTTP_COOKIE'}) {
 		# Different states of the client are stored in different caches
 		# Don't put different Google Analytics in different caches, and anyway they
@@ -776,6 +787,7 @@ Set various options and override default values.
 	logger => $logger,
 	lint_content => 0,	# Pass through HTML::Lint
 	generate_304 => 1,	# Generate 304: Not modified
+	lingua => CGI::Lingua->new(),
     );
 
 If no cache_key is given, one will be generated which may not be unique.
@@ -831,6 +843,9 @@ sub init {
 	if(defined($params{logger})) {
 		$logger = $params{logger};
 	}
+	if(defined($params{lingua})) {
+		$lingua = $params{lingua};
+	}
 	if(defined($params{generate_304})) {
 		$generate_304 = $params{generate_304};
 	}
@@ -1058,9 +1073,6 @@ sub _my_age {
 
 sub _should_gzip
 {
-	if(defined($send_body) && ($send_body == 0)) {
-		return '';
-	}
 	if($compress_content && ($ENV{'HTTP_ACCEPT_ENCODING'} || $ENV{'HTTP_TE'})) {
 		if(scalar(@content_type)) {
 			if($content_type[0] ne 'text') {
@@ -1124,7 +1136,7 @@ sub _compress {
 			require Encode;
 			$encode_loaded = 1;
 		}
-		my $nbody = bro(Encode::encode_utf8($body));
+		my $nbody = IO::Compress::Brotli::bro(Encode::encode_utf8($body));
 		if(length($nbody) < length($body)) {
 			$body = $nbody;
 			push @o, "Content-Encoding: $encoding";
@@ -1231,7 +1243,7 @@ The licence for cgi_buffer is:
 
     This software is provided 'as is' without warranty of any kind."
 
-The rest of the program is Copyright 2011-2016 Nigel Horne,
+The rest of the program is Copyright 2011-2017 Nigel Horne,
 and is released under the following licence: GPL
 
 =cut
